//
//  GMLegendDataSource.m
//  GIS
//
//  Created by Hamza Haroon on 12-09-07.
//  Copyright (c) 2012 Aecom. All rights reserved.
//

#import "GMLegendDataSource.h"

@implementation GMLegendDataSource

@synthesize legendInfos=_legendInfos;
@synthesize visibleIcons = _visibleIcons;


- (id)init {
    self = [super init];
    if (self) {
		//Initialize member variables.
		//legendInfos will hold objects containing information for each legend item
		self.legendInfos = [[[NSMutableArray alloc] init] autorelease];
        self.dynamicLayer = [[[AGSDynamicMapServiceLayer alloc] init] autorelease];
        self.visibleIcons = [[[NSMutableArray alloc] init]autorelease];
    }
    return self;
}

- (void) addLegendForLayer:(AGSLayer *)layer{// withView:(UIView *)view{
    //self.legendInfos = [[[NSMutableArray alloc] init]autorelease];
    //Check type of layer
	if ([layer isKindOfClass: [AGSDynamicMapServiceLayer class]]) {
		
		//Get the service info
		AGSMapServiceInfo* msi =  ((AGSDynamicMapServiceLayer*)layer).mapServiceInfo;
        
		//Legend only supported for services from  10 Service Pack 1 or above
		if(msi.version>=10.01){
			msi.delegate = self;
            
            LegendInfo *info = [[LegendInfo alloc] init];
            info.name = layer.name;
            info.detail = layer.name;
            info.image = nil;
            //info.view = view;
            
			[msi retrieveLegendInfo];
            //[self.legendInfos addObject:info];
		}else {
			NSLog(@"Skipping layer [%@]. ArcGIS Service must be version 10 SP1 or above",msi.URL );
		}
        
		
	}else if ([layer isKindOfClass: [AGSTiledMapServiceLayer class]]) {
		//Get the service info
		AGSMapServiceInfo* msi =  ((AGSTiledMapServiceLayer*)layer).mapServiceInfo;
        
		//Legend only supported for services from  10 Service Pack 1 or above
		if(msi.version>=10.01){
			msi.delegate = self;
			[msi retrieveLegendInfo];
            
		} else {
			NSLog(@"Skipping layer [%@]. ArcGIS Service must be version 10 SP1 or above",msi.URL);
		}
        
	}else if ([layer isKindOfClass: [AGSFeatureLayer class]]) {
		AGSFeatureLayer* featLayer = (AGSFeatureLayer*)layer;
        
		//Loop through the Types in feature layer
		NSArray* featTypes = featLayer.types;
		for (int i=0; i<[featTypes count]; i++) {
			AGSFeatureType* type = (AGSFeatureType*)[featTypes objectAtIndex:i];
			
			//Get a Template for the Type
			AGSFeatureTemplate* template = [type.templates objectAtIndex:0];
			
			//Get information of Template to display as a legend item
			LegendInfo* info = [[[LegendInfo alloc] init] autorelease];
			info.image = [featLayer.renderer swatchForGraphic:template.prototype geometryType:featLayer.geometryType size:CGSizeMake(20, 20)];
			info.name = type.name;
			[self.legendInfos addObject:info];
		}
		//Loop through the Templates in feature layer
		NSArray* featTemplates = featLayer.templates;
		for (int i=0; i<[featTemplates count]; i++) {
			AGSFeatureTemplate* template = (AGSFeatureTemplate*) [featTemplates objectAtIndex:i];
            
			//Get information of Template to display as a legend item
			LegendInfo* info = [[[LegendInfo alloc] init] autorelease];
			info.image = [featLayer.renderer swatchForGraphic:template.prototype geometryType:featLayer.geometryType size:CGSizeMake(20, 20)];
			info.name = template.name;
			[self.legendInfos addObject:info];
		}
        
		//Reload the table to display newly added legend items
		[self reload];
	}else {
		//skip other layer types
		//legend not supported
		NSLog(@"Skipping layer of type %@. Legend not supported.",[layer class]);
	}
}

#pragma mark -
#pragma mark AGSMapServiceInfoDelegate
- (void)mapServiceInfo:(AGSMapServiceInfo *)mapServiceInfo operationDidRetrieveLegendInfo:(NSOperation*)op {
	NSArray* layerInfos = mapServiceInfo.layerInfos;
	//Loop through all sub-layers
    //NSLog(@"visible icons %@", self.visibleIcons);
	for (int i=0; i<[layerInfos count]; i++)
    {
		
		AGSMapServiceLayerInfo* layerInfo = (AGSMapServiceLayerInfo*)[layerInfos objectAtIndex:i];
		NSArray* legendLabels = layerInfo.legendLabels;
		NSArray* legendImages = layerInfo.legendImages;
        LegendInfo* info = [[LegendInfo alloc] init];//autorelease];

        for(int j=0;j<[legendImages count];j++)
        {
			//Store info for each legend item
			LegendInfo* legendInfo = [[[LegendInfo alloc] init ]autorelease];
            if (j == 0)
            {
                legendInfo.name = layerInfo.name;
            }
			legendInfo.detail = [legendLabels objectAtIndex:j];
			legendInfo.image = [legendImages objectAtIndex:j];
            
            if ([self.visibleIcons containsObject:[AGSLayerDefinition layerDefinitionWithLayerId:layerInfo.layerId definition:legendInfo.detail]]) {
                legendInfo.visible = NO;
                
            }
            else
                legendInfo.visible = YES;
            
            legendInfo.layerID = (NSUInteger *)layerInfo.layerId;
            
			[self.legendInfos addObject:legendInfo];
		}
        /*
        if (!info.visible) {
            int layerID = (int)info.layerID;
            NSString *def = info.detail;
            AGSLayerDefinition *layerDef = [AGSLayerDefinition layerDefinitionWithLayerId:layerID definition:def];
            if (![self.visibleIcons containsObject:layerDef]) {
                [self.visibleIcons addObject:layerDef];
            }
        }*/
        
    }
}

- (void)mapServiceInfo:(AGSMapServiceInfo *)mapServiceInfo operation:(NSOperation*)op didFailToRetrieveLegendInfoWithError:(NSError*)error{
	NSLog(@"Error encountered while fetching legend : %@",error);
}


- (void) reload {
	[_tableView performSelectorOnMainThread:@selector(reloadData) withObject:nil waitUntilDone:NO];
}



- (void)dealloc {
	[super dealloc];}

@end

//A convenience class to hold information about each legend item
@implementation LegendInfo
@synthesize image = _image,name=_name,detail=_detail, visible, view, layerID = _layerID;

- (void)dealloc {
    self.image = nil;
	self.name = nil;
	self.detail = nil;
    self.view = nil;
    self.layerID = nil;
    [super dealloc];
}

@end
