//
//  GMViewController.m
//  GIS
//
//  Created by Hamza Haroon on 12-08-24.
//  Copyright (c) 2012 Aecom. All rights reserved.
//

#import "GMViewController.h"

@implementation GMViewController

@synthesize mapView = _mapView;
@synthesize tiledLayer;
@synthesize dynamicLayer;
@synthesize sketchLayer;
@synthesize sketchLayer2;
@synthesize graphicsLayer;
@synthesize graphicsLayer2;

@synthesize dynamicLayerView;
@synthesize backgroundLayerView;

@synthesize midSymbol;
@synthesize vertSymbol;
@synthesize cred;
@synthesize toolbar = _toolbar;
@synthesize identifyParams = _identifyParams;
@synthesize identifyTask = _identifyTask;
@synthesize mappoint = _mappoint;
@synthesize resultsVC;
@synthesize legendDataSource = _legendDataSource;
@synthesize visibleLegend;
@synthesize popOverController;

@synthesize graphicsLayer3;

- (void)viewDidLoad
{
    [super viewDidLoad];

    [[NSNotificationCenter defaultCenter] postNotificationName:@"CurrentViewChanged" object:self];
       
    self.mapView.layerDelegate = self;
    
    self.legendDataSource = [[GMLegendDataSource alloc] init] ;

    NSString *backgroundLayer = kTiledMapServiceURL1;
        
    AGSCredential *temp = [[AGSCredential alloc] initWithUser:nil password:nil];
    tiledLayer = [[AGSTiledMapServiceLayer alloc] initWithURL:[NSURL URLWithString:backgroundLayer] credential:temp ];

    self.backgroundLayerView =[self.mapView addMapLayer:tiledLayer withName:@"Tiled Layer"];
     
    self.dynamicLayer = [[AGSDynamicMapServiceLayer alloc] initWithURL:[NSURL URLWithString: kDynamicMapServiceURL1] credential:temp];
    
    self.dynamicLayerView =[self.mapView addMapLayer:self.dynamicLayer withName:@"Dynamic Layer"];

    [self loadMap];
    
    self.findTask = [[AGSFindTask alloc] initWithURL:self.dynamicLayer.URL credential:self.dynamicLayer.credential];
    self.findTask.delegate = self;
    
    self.findParams = [[AGSFindParameters alloc] init] ;
    self.searchBar.delegate = self;

}

-(void)viewDidAppear:(BOOL)animated{
    [[NSNotificationCenter defaultCenter] postNotificationName:@"CurrentViewChanged" object:self];

}

-(void) loadMap
{    
    //Graphics layer to hold all sketches (points, polylines, and polygons)
    graphicsLayer = [AGSGraphicsLayer graphicsLayer];
	[self.mapView addMapLayer:graphicsLayer withName:@"Graphics Layer"];
    
    graphicsLayer2 = [AGSGraphicsLayer graphicsLayer];
    [self.mapView addMapLayer:graphicsLayer2 withName:@"Graphics Layer 2"];
    
    graphicsLayer3 = [AGSGraphicsLayer graphicsLayer];
    [self.mapView addMapLayer:graphicsLayer3 withName:@"Graphics Layer 3"];
    
	//A composite symbol for the graphics layer's renderer to symbolize the sketches
	AGSCompositeSymbol* composite = [AGSCompositeSymbol compositeSymbol];
	AGSSimpleLineSymbol* lineSymbol = [[AGSSimpleLineSymbol alloc] init] ;
	lineSymbol.color= [UIColor redColor];
	lineSymbol.width = 4;
	[composite.symbols addObject:lineSymbol];
	AGSSimpleFillSymbol* fillSymbol = [[AGSSimpleFillSymbol alloc] init] ;
	fillSymbol.outline.color = [UIColor redColor];//colorWithRed:1.0 green:0 blue:0 alpha:4] ;
    fillSymbol.outline.width = 4;
	[composite.symbols addObject:fillSymbol];
	AGSSimpleRenderer* renderer = [AGSSimpleRenderer simpleRendererWithSymbol:composite];
    

    AGSSpatialReference *sr = [[AGSSpatialReference alloc] initWithWKID:4269];
    AGSEnvelope *env = [[AGSEnvelope alloc] initWithXmin:-185 ymin:15 xmax:-59 ymax:74 spatialReference:sr];
    
    [self.mapView zoomToEnvelope:env animated:YES];    
    
    graphicsLayer2.renderer = renderer;
    
	//Sketch layer
	sketchLayer = [[AGSSketchGraphicsLayer alloc] initWithGeometry:nil];
	[self.mapView addMapLayer:sketchLayer withName:@"Sketch layer"];
    
    self.sketchLayer2 = [[AGSSketchGraphicsLayer alloc] initWithGeometry:nil];
    self.sketchLayer2.geometry = [[AGSMutablePolyline alloc] initWithSpatialReference:self.mapView.spatialReference];
    
    [self.mapView addMapLayer:sketchLayer2 withName:@"Sketch layer2"];
	
    poly = [[AGSMutablePolyline alloc] initWithSpatialReference:self.mapView.spatialReference];
    polyLines = [[NSMutableArray alloc] init];
    
    self.sketchLayer.midVertexSymbol = nil;
    self.sketchLayer.vertexSymbol = nil;
        
    self.mapView.touchDelegate = self;
    self.mapView.calloutDelegate = self;
    
    self.identifyTask = [AGSIdentifyTask identifyTaskWithURL:self.dynamicLayer.URL credential:self.dynamicLayer.credential];
    self.identifyTask.delegate = self;
    
    self.identifyParams = [[AGSIdentifyParameters alloc] init] ;
}

//////////////// Segue controll /////////////

-(void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if (self.popOverController)
        {
            NSLog(@"popover %@", self.popOverController);
            [self.popOverController dismissPopoverAnimated:NO];
            self.popOverController = nil;
        }
    
    if ([segue.identifier isEqualToString:@"CalloutDetails"])
    {
        GMCalloutController *myCR = segue.destinationViewController;
        myCR.results = resultsVC;
    }
    else if ([segue.identifier isEqualToString:@"callLegend"])
    {
        legendInformation = [self.legendDataSource legendInfos];
        
        GMLegendController *vc = segue.destinationViewController;
        
        vc.delegate = self;
        vc.legendDataSource.legendInfos = self.legendDataSource.legendInfos;
        vc.legendDataSource.visibleIcons = self.visibleLegend;
        
        popOverController = [(UIStoryboardPopoverSegue *)segue popoverController];
    }
}

//if there's an error with the find display it to the user
-(void)findTask:(AGSFindTask *)findTask operation:(NSOperation *)op didFailWithError:(NSError *)error {
    
	UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error"
													message:[error localizedDescription]
												   delegate:nil
										  cancelButtonTitle:@"OK"
										  otherButtonTitles:nil];
	[alert show];
	//[alert release];
}

//////////////// CALLOUTS/////////////////////

#pragma mark - AGSMapViewCalloutDelegate methods

- (void)mapView:(AGSMapView *)mapView didClickAtPoint:(CGPoint)screen mapPoint:(AGSPoint *)mappoint graphics:(NSDictionary *)graphics {
    
    //store for later use
    self.mappoint = mappoint;
    
	self.identifyParams.tolerance = 15;
	self.identifyParams.geometry = self.mappoint;
	self.identifyParams.size = self.mapView.bounds.size;
	self.identifyParams.mapEnvelope = self.mapView.visibleArea.envelope;
	self.identifyParams.returnGeometry = YES;
	self.identifyParams.layerOption = AGSIdentifyParametersLayerOptionTop;
	self.identifyParams.spatialReference = self.mapView.spatialReference;
    
	//execute the task
	[self.identifyTask executeWithParameters:self.identifyParams];
	
}

//show the attributes if accessory button is clicked
- (void)mapView:(AGSMapView *)mapView didClickCalloutAccessoryButtonForGraphic: (AGSGraphic *) 	graphic {
    
    resultsVC = graphic.attributes;
    
    //display the results vc modally
    [self performSegueWithIdentifier:@"CalloutDetails" sender:self];
}

#pragma mark - AGSIdentifyTaskDelegate methods
//results are returned
- (void)identifyTask:(AGSIdentifyTask *)identifyTask operation:(NSOperation *)op didExecuteWithIdentifyResults:(NSArray *)results {
	
    //clear previous results
    [graphicsLayer2 removeAllGraphics];
	
    //add new results
    AGSSymbol* symbol = [AGSSimpleFillSymbol simpleFillSymbol];
    symbol.color = [UIColor colorWithRed:0 green:0 blue:1 alpha:0.5];
	
	// for each result, set the symbol and add it to the graphics layer
    for (AGSIdentifyResult* result in results) {
        result.feature.symbol = symbol;
        // [_graphicsLayer addGraphic:result.feature];
    }
    
    if ([results count]  != 0 ) {
        //set the callout content for the first result
        //get the state name
        NSString *stateNameP1 = [((AGSIdentifyResult*)[results objectAtIndex:0]) layerName];
        NSString *stateNameP2 = [((AGSIdentifyResult*)[results objectAtIndex:0]).feature.attributes objectForKey:@"NAME"];
        NSString *imageURL = [((AGSIdentifyResult*)[results objectAtIndex:0]).feature.attributes objectForKey:@"LINK"];
        if (imageURL != nil)
        {
            NSURL *imageURLObject = [NSURL URLWithString:imageURL];
            //[self.photoViewController loadUrlWithRepeatInterval:imageURLObject withRepeatInterval:10];
            //self.mapView.callout.customView = self.photoViewController.view;
            [self.mapView showCalloutAtPoint:self.mappoint forGraphic:((AGSIdentifyResult*)[results objectAtIndex:0]).feature animated:YES];
        }
        else
        {
            self.mapView.callout.customView = nil;
            if (stateNameP2 != nil)
            {
                NSString *stateName = [stateNameP1 stringByAppendingString:[@"  "stringByAppendingString:stateNameP2]];
                self.mapView.callout.title = stateName;
                self.mapView.callout.detail = @"Click for more detail..";
            }
            else
            {
                NSString *stateName =stateNameP1;
                self.mapView.callout.title = stateName;
                self.mapView.callout.detail = @"Click for more detail..";
            }
            //show callout
            [self.mapView showCalloutAtPoint:self.mappoint forGraphic:((AGSIdentifyResult*)[results objectAtIndex:0]).feature animated:YES];
        }
        
        
        //call dataChanged on the graphics layer to redraw the graphics
        [graphicsLayer2 dataChanged];
    }
    
}

//if there's an error with the query display it to the user
- (void)identifyTask:(AGSIdentifyTask *)identifyTask operation:(NSOperation *)op didFailWithError:(NSError *)error {
	NSLog(@"error %@", [error localizedFailureReason]);
	UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error"
													message:[error localizedFailureReason]//localizedDescription]
												   delegate:nil
										  cancelButtonTitle:@"OK"
										  otherButtonTitles:nil];
	[alert show];
	//[alert release];
}
#pragma mark

//////////////////////////////////////////////

///////////// LEGEND /////////////////////////

- (void)mapView:(AGSMapView *) mapView didLoadLayerForLayerView:(UIView<AGSLayerView> *) layerView {
	//Add legend for each layer added to the map
	[self.legendDataSource addLegendForLayer:(AGSLayer *)layerView.agsLayer];
}

- (IBAction)showLegend:(id)sender {
     
     [self performSegueWithIdentifier:@"callLegend" sender:self];
}

-(void)reloadMap:(NSMutableArray *)hiddenIcons
{
    self.visibleLegend = hiddenIcons;
    for (int i = 0; i < [visibleLegend count]; i++) {
        for (int j= 0; j < [visibleLegend count]; j++)
        {
            AGSLayerDefinition *layer1 = [visibleLegend objectAtIndex:i];
            AGSLayerDefinition *layer2 = [visibleLegend objectAtIndex:j];
            if (i != j)
            {
                if (layer1.layerId == layer2.layerId) {
                    [self.visibleLegend removeObjectAtIndex:j];
                }

            }
        }
        
        
    }
    self.dynamicLayer.layerDefinitions = self.visibleLegend;
}

/////////////////////////////////////////////

-(void) mapView:(AGSMapView *) mapView failedLoadingLayerForLayerView:(UIView<AGSLayerView> *)layerView baseLayer:(BOOL)baseLayer withError:(NSError *)error
{
    if (!baseLayer)
        [self.mapView removeMapLayerWithName:layerView.name];
}

- (void)viewDidUnload
{
    [self setMapView:nil];

    [super viewDidUnload];
    // Release any retained subviews of the main view.
}

-(void)dealloc
{
    self.mapView = nil;
    self.toolbar = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

@end
